/*------------------------------------------------------------------------------
--                                                                            --
--           This software is confidential and proprietary and may be used    --
--              only as expressly authorized by a licensing agreement from    --
--                                                                            --
--                            Hantro Products Oy.                             --
--                                                                            --
--      In the event of publication, the following notice is applicable:      --
--                                                                            --
--                   (C) COPYRIGHT 2001 HANTRO PRODUCTS OY                    --
--                            ALL RIGHTS RESERVED                             --
--                                                                            --
--    The entire notice above must be reproduced on all copies.               --
--                                                                            --
--------------------------------------------------------------------------------
--
--  Description : Encoder internal
--
--------------------------------------------------------------------------------
--
--  Version control information, please leave untouched.
--
--  $RCSfile: EncPreProcess.h,v $
--  $Date: 2005/11/04 06:59:38 $
--  $Revision: 1.10 $
--
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------

    Table of context

    1. Include headers
    2. External compiler flags
    3. Module defines
    4. Function prototypes

------------------------------------------------------------------------------*/
#ifndef __ENC_PRE_PROCESS_H__
#define __ENC_PRE_PROCESS_H__

/*------------------------------------------------------------------------------
    1. Include headers
------------------------------------------------------------------------------*/
#include "basetype.h"
#include "EncSim.h"

/*------------------------------------------------------------------------------
    2. External compiler flags
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
    3. Module defines
------------------------------------------------------------------------------*/

#undef PTRACE			/* undef it, just in case */

#if defined(__GNUC__)
#   if defined(TRACE_PP)
#       define PTRACE(fmt, args...) printf(__FILE__ ":%d: " fmt, __LINE__ , ## args)
#   else
#       define PTRACE(fmt, args...)	/* no trace */
#   endif
#else
#   define PTRACE /* no trace */
#endif


typedef struct {
	true_e smooth;		/* smoothing enable */
	i32 stabFrame;		/* stab size; 0 to disable */

	i32 lumWidth;
	i32 lumHeight;

	i32 lumWidthSrc;	/* Lum width of source (input) image */
	i32 lumHeightSrc;	/* Lum height of source (input) image */

	i32 chWidthSrc;

	i32 horOffsetSrc;	/* Stabilizator hor. offset */
	i32 verOffsetSrc;	/* and ver. offset */
	i32 horOffset;		/* Output frame offset, reference is ... */
	i32 verOffset;		/* ...top left corner of stabilator frame */

	i32 horSpeed;
	i32 verSpeed;

	u32 busAddrLumaY;
	u32 busAddrChromaU;
	u32 busAddrChromaV;
} preProcess_s;

/*------------------------------------------------------------------------------
    4. Function prototypes
------------------------------------------------------------------------------*/
bool_e EncPreProcessCheck(preProcess_s * preProcess);
void EncStabilization(preProcess_s * preProcess);
void EncPreProcess(preProcess_s * preProcess);

#endif
