/*------------------------------------------------------------------------------
--                                                                            --
--       This software is confidential and proprietary and may be used        --
--        only as expressly authorized by a licensing agreement from          --
--                                                                            --
--                            Hantro Products Oy.                             --
--                                                                            --
--      In the event of publication, the following notice is applicable:      --
--                                                                            --
--                   (C) COPYRIGHT 2003 HANTRO PRODUCTS OY                    --
--                            ALL RIGHTS RESERVED                             --
--                                                                            --
--          The entire notice above must be reproduced on all copies.         --
--                                                                            --
--------------------------------------------------------------------------------
--
--  Abstract : Encoder HW register access functions
--
--------------------------------------------------------------------------------
--
--  Version control information, please leave untouched.
--
--  $RCSfile: encregdrv1.c,v $
--  $Date: 2005/12/20 16:21:35 $
--  $Revision: 1.12 $
--
------------------------------------------------------------------------------*/
#include "ewl.h"
#include "basetype.h"
#include "encregdrv.h"
#include "EncSim.h"

/* 1. (AMBAIFE) Encoder Register Offsets   */

#define ENCREG(nr)	(nr<<2)

/* 2. Mask fields */
#define mask_1b         (u32)0x00000001
#define mask_2b         (u32)0x00000003
#define mask_3b         (u32)0x00000007
#define mask_4b         (u32)0x0000000F
#define mask_5b         (u32)0x0000001F
#define mask_7b         (u32)0x0000007F
#define mask_9b         (u32)0x000001FF
#define mask_10b        (u32)0x000003FF
#define mask_16b        (u32)0x0000FFFF

/* 3. Encoder register access drivers */

/******************************************************************************/
/********************** ENCODER REGISTER (BASE + 0x00) ************************/
/******************************************************************************/

/*******************************************************************************
 Function name   : SetEncQP
 Description     : Set the first quantization parameter
 Return type     : void 
 Argument        : u32 qp
*******************************************************************************/
void SetEncQP(u32 qp)
{
    u32 val;

    ASSERT(qp > 0 && qp <= 31);
    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_5b << 17)) | (qp << 17);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncQP
 Description     : Get the first quantization parameter
 Return type     : u32 
*******************************************************************************/
u32 GetEncQP()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (val >> 17) & mask_5b;
    return val;
}

/*******************************************************************************
 Function name   : SetEncIntraDcVlcThr
 Description     : Set the threshold value for intra/inter vlc switch for DC
 Return type     : void 
 Argument        : u32 thr
*******************************************************************************/
void SetEncIntraDcVlcThr(u32 thr)
{
    u32 val;

    ASSERT(thr <= 7);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_3b << 14)) | (thr << 14);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncIntraDcVlcThr
 Description     : Get the threshold value for intra/inter vlc switch for DC
 Return type     : u32 
*******************************************************************************/
u32 GetEncIntraDcVlcThr()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = mask_3b & (val >> 14);
    return val;
}

/*******************************************************************************
 Function name   : SetEncDataPEnable
 Description     : Enable/Disable data partitioning
 Return type     : void 
 Argument        : u32 enable
*******************************************************************************/
void SetEncDataPEnable(u32 enable)
{
    u32 val;

    ASSERT(enable <= 1);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_1b << 13)) | (enable << 13);

    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncDataPEnable
 Description     : Get data partitioning enable status
 Return type     : u32 
*******************************************************************************/
u32 GetEncDataPEnable()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = mask_1b & (val >> 13);
    return val;
}

/*******************************************************************************
 Function name   : SetEncSVHEnable
 Description     : Enable/Disable SVH mode
 Return type     : void 
 Argument        : u32 enable
*******************************************************************************/
void SetEncSVHEnable(u32 enable)
{
    u32 val;

    ASSERT(enable <= 1);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_1b << 11)) | (enable << 11);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncSVHEnable
 Description     : Get SVH enable status
 Return type     : u32 
*******************************************************************************/
u32 GetEncSVHEnable()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = mask_1b & (val >> 11);
    return val;
}

/*******************************************************************************
 Function name   : SetEncAcPredEnable
 Description     : Enable the AC prediction
 Return type     : void 
 Argument        : u32 mode
*******************************************************************************/
void SetEncAcPredEnable(u32 enable)
{
    u32 val;

    ASSERT(enable <= 1);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_1b << 6)) | (enable << 6);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncAcPredEnable
 Description     : Get the AC prediction status
 Return type     : u32 
*******************************************************************************/
u32 GetEncAcPredEnable()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (mask_1b & (val >> 6));
    return val;
}

/*******************************************************************************
 Function name   : SetEncOutsideVopMvEnable
 Description     : Enable/Disable MV pointing out of the picture
 Return type     : void 
 Argument        : u32 enable
*******************************************************************************/
void SetEncOutsideVopMvEnable(u32 enable)
{
    u32 val;

    ASSERT(enable <= 1);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_1b << 7)) | (enable << 7);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncOutsideVopMvEnable
 Description     : Get MV pointing out of the picture enable status
 Return type     : u32 
*******************************************************************************/
u32 GetEncOutsideVopMvEnable()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (mask_1b & (val >> 7));
    return val;
}

/*******************************************************************************
 Function name   : SetEncRvlcEnable
 Description     : Enable/Disable RVLC usage
 Return type     : void 
 Argument        : u32 enable
*******************************************************************************/
void SetEncRvlcEnable(u32 enable)
{
    u32 val;

    ASSERT(enable <= 1);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_1b << 4)) | (enable << 4);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncRvlcEnable
 Description     : Get RVLC enable status
 Return type     : u32 
*******************************************************************************/
u32 GetEncRvlcEnable()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (mask_1b & (val >> 4));
    return val;
}

/*******************************************************************************
 Function name   : SetEncRoundingCtrl
 Description     : Set the rounding value used in motion compensation
 Return type     : void 
 Argument        : u32 roundval
*******************************************************************************/
void SetEncRoundingCtrl(u32 roundval)
{
    u32 val;

    ASSERT(roundval <= 1);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_1b << 3)) | (roundval << 3);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncRoundingCtrl
 Description     : Get the rounding value used in motion compensation
 Return type     : u32 
*******************************************************************************/
u32 GetEncRoundingCtrl()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (mask_1b & (val >> 3));
    return val;
}

/*******************************************************************************
 Function name   : SetEncStrmEndianMode
 Description     : Set the output stream endian mode
 Return type     : void 
 Argument        : u32 endian
*******************************************************************************/
void SetEncStrmEndianMode(u32 endian)
{
    u32 val;

    ASSERT(endian <= 1);

    val = EWL_ReadReg(0x00);
    val = (val & (~(mask_1b<<5))) | (endian<<5);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : SetEncStrmEndianMode
 Description     : Get the output stream endian mode
 Return type     : u32 
*******************************************************************************/
u32 GetEncStrmEndianMode()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (mask_1b & (val >> 5));
    return val;
}

/*******************************************************************************
 Function name   : SetEncImgEndianMode
 Description     : Set the input image endian mode
 Return type     : void 
 Argument        : u32 endian
*******************************************************************************/
void SetEncImgEndianMode(u32 endian)
{
    u32 val;

    ASSERT(endian <= 1);

    val = EWL_ReadReg(0x00);
    val = (val & ~mask_1b) | endian;
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncImgEndianMode
 Description     : Get the input image endian mode
 Return type     : u32 
*******************************************************************************/
u32 GetEncImgEndianMode()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (mask_1b & val);
    return val;
}

/*******************************************************************************
 Function name   : SetEncModTimeBase
 Description     : Set the modulo time base
 Return type     : void 
 Argument        : u32 sec
*******************************************************************************/
void SetEncModTimeBase(u32 sec)
{
    u32 val;

    ASSERT(sec <= 127);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_7b << 25)) | (sec << 25);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncModTimeBase
 Description     : Get the current modulo time base value
 Return type     : u32 
*******************************************************************************/
u32 GetEncModTimeBase()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (val >> 25);
    return val;
}

/*******************************************************************************
 Function name   : SetEncGobFrmId
 Description     : Set the GOB frame identification field in H.263(SVH) mode
 Return type     : void 
 Argument        : u32 id
*******************************************************************************/
void SetEncGobFrmId(u32 id)
{
    u32 val;

    ASSERT(id <= 3);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_2b << 8)) | (id << 8);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncGobFrmId
 Description     : Get the current GOB frame identification field
 Return type     : u32 
*******************************************************************************/
u32 GetEncGobFrmId()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (mask_2b & (val >> 8));
    return val;
}

/*******************************************************************************
 Function name   : SetEncGobRows
 Description     : Set the amount of rows in a GOB
 Return type     : void 
 Argument        : u32 rows
                   0    GOB is 1 row (frame height <= 25 macroblocks)
                   1    GOB is 2 rows (frame height > 25 macroblocks)
*******************************************************************************/
void SetEncGobRows(u32 rows)
{
    u32 val;

    ASSERT(rows <= 1);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_1b << 10)) | (rows << 10);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncGobRows
 Description     : Get the current status of rows in a GOB
 Return type     : u32 
*******************************************************************************/
u32 GetEncGobRows()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (mask_1b & (val >> 10));
    return val;
}

/*******************************************************************************
 Function name   : SetEncVopType
 Description     : Set the VOP encoding type
 Return type     : void 
 Argument        : u32 type
*******************************************************************************/
void SetEncVopType(u32 type)
{
    u32 val;

    ASSERT(type <= 2);

    val = EWL_ReadReg(0x00);
    val = (val & ~(mask_2b << 1)) | (type << 1);
    EWL_WriteReg(0x00, val);
}

/*******************************************************************************
 Function name   : GetEncVopType
 Description     : Get the current VOP type
 Return type     : u32 
*******************************************************************************/
u32 GetEncVopType()
{
    u32 val;

    val = EWL_ReadReg(0x00);
    val = (mask_2b & (val >> 1));
    return val;
}

/******************************************************************************/
/********************** ENCODER REGISTER (BASE + 0x04) ************************/
/******************************************************************************/

/*******************************************************************************
 Function name   : SetEncPreProEnable
 Description     : Enable/disable preprocessing block
 Return type     : void 
 Argument        : u32 enable
*******************************************************************************/
void SetEncPreProEnable(u32 enable)
{
    u32 val;

    ASSERT(enable <= 1);

    val = EWL_ReadReg(0x04);
    val = (val & ~(mask_1b << 31)) | (enable << 31);
    EWL_WriteReg(0x04, val);
}

/*******************************************************************************
 Function name   : GetEncPreProEnable
 Description     : Get the current preprocessing block status
 Return type     : u32 
*******************************************************************************/
u32 GetEncPreProEnable()
{
    u32 val;

    val = EWL_ReadReg(0x04);
    val = (val >> 31);
    return val;
}

/*******************************************************************************
 Function name   : SetEncOrigRowWidth
 Description     : Set the row width of the original input image (pixelWidth/8)
 Return type     : void 
 Argument        : u32 width
*******************************************************************************/
void SetEncOrigRowWidth(u32 width)
{
    u32 val;

    ASSERT(width >= 12 && width <= 127);

    val = EWL_ReadReg(0x04);
    val = (val & ~(mask_7b << 21)) | (width << 21);
    EWL_WriteReg(0x04, val);
}

/*******************************************************************************
 Function name   : GetEncOrigRowWidth
 Description     : Get the row width of the original input image
 Return type     : u32 
*******************************************************************************/
u32 GetEncOrigRowWidth()
{
    u32 val;

    val = EWL_ReadReg(0x04);
    val = (mask_7b & (val >> 21));
    return val;
}

/*******************************************************************************
 Function name : SetEncXFill
 Description   : Set how many pixel columns shall be filled with constant
                 values at the right end of picture
 Return type   : void 
 Argument      : u32 xfill
                 0  Fill 0 pixels
                 1  Fill 4 pixels
                 2  Fill 8 pixels
                 3  Fill 12 pixels
*******************************************************************************/
void SetEncXFill(u32 xfill)
{
    u32 val;

    ASSERT(xfill <= 3);
    val = EWL_ReadReg(0x04);
    val = (val & ~(mask_2b << 19)) | (xfill << 19);
    EWL_WriteReg(0x04, val);
}

/*******************************************************************************
 Function name : GetEncXFill
 Description   : 
 Return type   : u32 
 Argument      : void
*******************************************************************************/
u32 GetEncXFill()
{
    u32 val;

    val = EWL_ReadReg(0x04);
    val = (mask_2b & (val >> 19));
    return val;
}

/*******************************************************************************
 Function name : SetEncYFill
 Description   : Set how many pixel lines shall be filled with constant
                 values at the bottom of the picture
 Return type   : void 
 Argument      : u32 yfill
*******************************************************************************/
void SetEncYFill(u32 yfill)
{
    u32 val;

    ASSERT(yfill <= 15);
    val = EWL_ReadReg(0x04);
    val = (val & ~(mask_4b << 15)) | (yfill << 15);
    EWL_WriteReg(0x04, val);
}

/*******************************************************************************
 Function name : GetEncYFill
 Description   : 
 Return type   : u32 
 Argument      : void
*******************************************************************************/
u32 GetEncYFill()
{
    u32 val;

    val = EWL_ReadReg(0x04);
    val = (mask_4b & (val >> 15));
    return val;
}

/*******************************************************************************
 Function name   : SetEncMvSadPenalty
 Description     : Set the penalty value for non zero MV
 Return type     : void 
 Argument        : u32 penalty
*******************************************************************************/
void SetEncMvSadPenalty(u32 penalty)
{
    u32 val;

    ASSERT(penalty <= 31);

    val = EWL_ReadReg(0x04);
    val = (val & ~(mask_5b << 10)) | (penalty << 10);
    EWL_WriteReg(0x04, val);
}

/*******************************************************************************
 Function name   : GetEncMvSadPenalty
 Description     : Get the penalty value for non zero MV
 Return type     : u32 
*******************************************************************************/
u32 GetEncMvSadPenalty()
{
    u32 val;

    val = EWL_ReadReg(0x04);
    val = (mask_5b & (val >> 10));
    return val;
}

/*******************************************************************************
 Function name   : SetEncIntraSadPenalty
 Description     : Set the penalty value for INTRA macroblocks
 Return type     : void 
 Argument        : u32 penalty
*******************************************************************************/
void SetEncIntraSadPenalty(u32 penalty)
{
    u32 val;

    ASSERT(penalty <= 31);

    val = EWL_ReadReg(0x04);
    val = (val & ~(mask_5b << 5)) | (penalty << 5);
    EWL_WriteReg(0x04, val);
}

/*******************************************************************************
 Function name   : GetEncIntraSadPenalty
 Description     : Get the penalty value for INTRA macroblocks
 Return type     : u32 
*******************************************************************************/
u32 GetEncIntraSadPenalty()
{
    u32 val;

    val = EWL_ReadReg(0x04);
    val = (mask_5b & (val >> 5));
    return val;
}

/*******************************************************************************
 Function name   : SetEnc4MvSadPenalty
 Description     : Set the penalty value for INTER 4MV macroblocks
 Return type     : void 
 Argument        : u32 penalty
*******************************************************************************/
void SetEnc4MvSadPenalty(u32 penalty)
{
    u32 val;

    ASSERT(penalty <= 31);

    val = EWL_ReadReg(0x04);
    val = (val & ~mask_5b) | penalty;
    EWL_WriteReg(0x04, val);
}

/*******************************************************************************
 Function name   : GetEnc4MvSadPenalty
 Description     : Get the penalty value for INTER 4MV macroblocks
 Return type     : u32 
*******************************************************************************/
u32 GetEnc4MvSadPenalty()
{
    u32 val;

    val = EWL_ReadReg(0x04);
    val = val & mask_5b;
    return val;
}

/******************************************************************************/
/********************** ENCODER REGISTER (BASE + 0x08) ************************/
/******************************************************************************/

/*******************************************************************************
 Function name   : SetEncCamStabEnable
 Description     : Enable the camera stabilization
 Return type     : void 
 Argument        : u32 enable
*******************************************************************************/
void SetEncCamStabEnable(u32 enable)
{
    u32 val;

    ASSERT(enable <= 1);

    val = EWL_ReadReg(0x08);
    val = (val & ~(mask_1b << 31)) | (enable << 31);
    EWL_WriteReg(0x08, val);
}

/*******************************************************************************
 Function name   : GetEncCamStabEnable
 Description     : Get the camera stabilization status
 Return type     : u32 
*******************************************************************************/
u32 GetEncCamStabEnable()
{
    u32 val;

    val = EWL_ReadReg(0x08);
    val = (val >> 31);
    return val;
}

/*******************************************************************************
 Function name   : SetEncVpEnable
 Description     : Enable/Disable VP generation
 Return type     : void 
 Argument        : u32 enable
*******************************************************************************/
void SetEncVpEnable(u32 enable)
{
    u32 val;

    ASSERT(enable <= 1);

    val = EWL_ReadReg(0x08);
    val = (val & ~(mask_1b << 28)) | (enable << 28);
    EWL_WriteReg(0x08, val);
}

/*******************************************************************************
 Function name   : GetEncVpEnable
 Description     : Get VP status
 Return type     : u32 
*******************************************************************************/
u32 GetEncVpEnable()
{
    u32 val;

    val = EWL_ReadReg(0x08);
    val = (mask_1b & (val >> 28));
    return val;
}

/*******************************************************************************
 Function name   : SetEncSizeOfVP
 Description     : Set the size of a VP
 Return type     : void 
 Argument        : u32 size
*******************************************************************************/
void SetEncSizeOfVP(u32 size)
{
    u32 val;

    ASSERT(size <= 65535);

    val = EWL_ReadReg(0x08);
    val = (val & ~(mask_16b << 12)) | (size << 12);
    EWL_WriteReg(0x08, val);
}

/*******************************************************************************
 Function name   : GetEncSizeOfVP
 Description     : Get the size of a VP
 Return type     : u32 
*******************************************************************************/
u32 GetEncSizeOfVP()
{
    u32 val;

    val = EWL_ReadReg(0x08);
    val = (mask_16b & (val >> 12));
    return val;
}

/*******************************************************************************
 Function name   : GetEncCamStabGmvX
 Description     : Get the general MV's horizontal component
 Return type     : u32 
*******************************************************************************/
u32 GetEncCamStabGmvX()
{
    u32 val;

    val = EWL_ReadReg(0x08);
    val = (mask_5b & (val >> 6));
    return val;
}

/*******************************************************************************
 Function name   : GetEncCamStabGmvY
 Description     : Get the general MV's vertical component
 Return type     : u32 
*******************************************************************************/
u32 GetEncCamStabGmvY()
{
    u32 val;

    val = EWL_ReadReg(0x08);
    val = (val & mask_5b);
    return val;
}
