#*****************************************************************************
#
# Mpeg4 Encoder  Library Makefile
#
#*****************************************************************************

# Library Name
PROGRAM		= mpeg4_encoder
CSRCS_DIR	= .
OBJ_DIR		= object
INC_DIR		= .


ifeq ($(BUILD),ARM11ELINUX)
	TOOL_CHAIN_PATH	= /vobs/fsl_mad_linux_toolchains/gcc-4.1.1-glibc-2.4-nptl-6/arm-none-linux-gnueabi
	TGT_OS_BIN	= arm11_ELINUX
	CC		= $(TOOL_CHAIN_PATH)/bin/arm-none-linux-gnueabi-gcc
	AR		= $(TOOL_CHAIN_PATH)/bin/arm-none-linux-gnueabi-ar
	LD		= $(TOOL_CHAIN_PATH)/bin/arm-none-linux-gnueabi-ld
	STDLIBDIR	= -L$(TOOL_CHAIN_PATH)/lib/gcc/arm-none-linux-gnueabi/4.1.1
endif

ifeq ($(BUILD),ARM9ELINUX)
	TOOL_CHAIN_PATH	= /vobs/fsl_mad_linux_toolchains/gcc-4.1.1-glibc-2.4-nptl-sf-1/arm-926ejs-linux
	TGT_OS_BIN	= arm9_ELINUX
	CC 		= $(TOOL_CHAIN_PATH)/bin/arm-926ejs-linux-gcc
	AR 		= $(TOOL_CHAIN_PATH)/bin/arm-926ejs-linux-ar
	LD 		= $(TOOL_CHAIN_PATH)/bin/arm-926ejs-linux-ld
        STDLIBDIR	= -L$(TOOL_CHAIN_PATH)/lib/gcc/arm-926ejs-linux-gnueabi/4.1.1
endif

ifeq ($(BUILD),UNIX)
	TGT_OS_BIN 	= UNIX
	CC 		= gcc
	AR 		= ar
	LD 		= ld
endif

LIBRARY     = lib$(PROGRAM)_$(TGT_OS_BIN)


ifeq ($(BUILD),UNIX)
	C_DEFINES  += -Wall  -D__NO_SCREEN__ -D_NO_OUTPUT_ -D_Embedded_x86 -D_LINUX_
else
	C_DEFINES  += -fPIC -Wall  -D__NO_SCREEN__ -D_NO_OUTPUT_ -D_Embedded_x86 -D_LINUX_ 
endif

ifeq ($(BUILD),UNIX)
	CARM	=  $(CFLAGS_$(BUILD)) $(C_DEFINES)
else
	CARM	=  $(CFLAGS_$(BUILD)) $(C_DEFINES)
endif

# Put the C files here
C_OBJS		= $(OBJ_DIR)/EncApi.o \
		$(OBJ_DIR)/encasic.o \
		$(OBJ_DIR)/EncGroupOfVideoObjectPlane.o \
		$(OBJ_DIR)/encinternalapi.o \
		$(OBJ_DIR)/encmemory.o \
		$(OBJ_DIR)/EncPreProcess.o \
		$(OBJ_DIR)/EncProfile.o \
		$(OBJ_DIR)/EncPutBits.o \
		$(OBJ_DIR)/EncRateControl.o \
		$(OBJ_DIR)/encregdrv1.o \
		$(OBJ_DIR)/encregdrv2.o \
		$(OBJ_DIR)/encregdrv3.o \
		$(OBJ_DIR)/encregdrv4.o \
		$(OBJ_DIR)/encregdrv5.o \
		$(OBJ_DIR)/encregdrv6.o \
		$(OBJ_DIR)/EncShortVideoHeader.o \
		$(OBJ_DIR)/encstrmforming.o \
		$(OBJ_DIR)/EncTimeCode.o \
		$(OBJ_DIR)/EncTrace.o \
		$(OBJ_DIR)/EncUserData.o \
		$(OBJ_DIR)/EncVideoObject.o \
		$(OBJ_DIR)/EncVideoObjectLayer.o \
		$(OBJ_DIR)/EncVideoObjectPlane.o \
		$(OBJ_DIR)/EncVisualObject.o \
		$(OBJ_DIR)/EncVisualObjectSequence.o \
		$(OBJ_DIR)/ewl.o \
		$(OBJ_DIR)/mp4encapi.o

#Include only 'c' files for unix build
ifeq ($(BUILD),UNIX)
	OBJS = $(C_OBJS)
else
	OBJS = $(C_OBJS)
endif

all: 		create  LIB_$(TGT_OS_BIN)
		@+echo "--- Build-all done ---"

# create the object directory
create:
		@echo "Creating directory" $(OBJDIR)
		@mkdir -p $(OBJ_DIR)

#Build the library

LIB_arm11_ELINUX	:$(OBJS)
			$(LD) -o $(LIB_ARGS) $(LIBRARY).so --shared -fPIC $(OBJS) $(STDLIBDIR) -lgcc

LIB_arm9_ELINUX		:$(OBJS)
		        $(LD) -o $(LIB_ARGS) $(LIBRARY).so --shared -fPIC $(OBJS) $(STDLIBDIR) -lgcc

LIB_UNIX		:$(OBJS)
			$(AR) -crv $(LIB_ARGS) $(LIBRARY).a $(OBJS)

$(OBJ_DIR)/%.o: 	$(CSRCS_DIR)/%.c
			$(CC) $(CARM) -I$(INC_DIR)  -c  -o $@ $<
			
			
clean_ARM11ELINUX:
		
		 rm -f $(OBJS) *~ core
		rm -f $(LIBRARY).so
		
clean_ARM9ELINUX:
		rm -f $(OBJS) *~ core
		rm -f $(LIBRARY).so
		
clean_UNIX:
		rm -f $(OBJS) *~ core
		rm -f $(LIBRARY).a

clean:
		rm -f $(OBJS) *~ core
		rm -f $(LIBRARY).so

usage:
	@echo "--------------------------------------------------------------"
	@echo "           Makefile for MP4  Parser Library "
	@echo "--------------------------------------------------------------"
	@echo ""
	@echo "Usage: $(MAKE) [options] [targets] [flags]"
	@echo "        flags:"
	@echo "            clean - clean all"
	@echo "            BUILD=UNIX clean  - clean UNIX"
	@echo "            BUILD=ARM9ELINUX  - clean ARM9ELINUX"
	@echo "            BUILD=ARM11ELINUX - clean ARM11ELINUX"
	@echo "            BUILD=UNIX - Builds for UNIX	"
	@echo "		 =ARM11ELINUX - builds for MX31 "
	@echo "		 =ARM9ELINUX  - builds for MX27 "
	@echo " "
	@echo "  For details of the options, please see the release notes"



